;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#xef04295c6a752ba6d68fc7622acff3e8 . "./code.cg2js/cg-status-bar.fasl")
(#x69cc1c2e3459f62815b0cd1473da36a4 . "./code.cg2js/cg-hotspot.fasl")
(#x080fb91debe447dd27bdacba05d6eafe . "./code.cg2js/cg-toolbar.fasl")
(#x904760f26c4b80c8e1236350f444d220 . "./code.cg2js/cg-clipboard-stack.fasl")
(#xbfe55fd295682310a894107d17cecd73 . "./code.cg2js/cg-tooltip.fasl")
(#x6fd28028c8bb60c308b08183f2bc6fb5 . "./code.cg2js/cg-calendar.fasl")
(#xdbb000965ba7719fe5ac3e37e56bcff8 . "./code.cg2js/cg-item-list.fasl")
(#x7a1e2dc0fa40630037593d236ea45f6f . "./code.cg2js/cg-bitmap-stream.fasl")
(#x34fa20af14ea9f5761c301044cdb5ede . "./code.cg2js/cg-trackbar.fasl")
(#x9ff6d29cade5ce670e0ad8c4f063ddcf . "./code.cg2js/cg-class-grid.fasl")
(#x39465762952e0615976c5c8024d66101 . "./code.cg2js/cg-dialog-item.fasl")
(#x648c18f1e1aa1d200a7c2a39913193b4 . "./code.cg2js/cg-comtab.fasl")
(#x47c925a5c14faff5fb6fb5505f9ed7a5 . "./code.cg2js/cg-lisp-text.fasl")
(#xae6ccf44b0b37f895c43415c43067a32 . "./code.cg2js/cg-rich-edit-pane.fasl")
(#x470570d0a7de5a44b1cabe7d0ab4b839 . "./code.cg2js/cg-button.fasl")
(#xecf653a266d9761d92168cc8bbd78967 . "./code.cg2js/cg-printing.fasl")
(#x101926fb7d09f7ec2edbf505068f6e47 . "./code.cg2js/cg-pixmap-printing.fasl")
(#xe65a4f362fd96f730e3ad8e26f8d6b86 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x4ae2fa8aaa02200f21b5614945d94b21 . "./code.cg2js/cg-pixmap-widget.fasl")
(#x0857355361748b97fd00c77979f86852 . "./code.cg2js/cg-selected-object.fasl")
(#x3968338b44a6d70a89be229d74263253 . "./code.cg2js/cg-text-or-combo.fasl")
(#xa64456aa5356dfe9af6fc979e9e35fc5 . "./code.cg2js/cg-graphics-context.fasl")
(#x119452d14912019f74f03267db416022 . "./code.cg2js/cg-edit-in-place.fasl")
(#xe20ffb6035ea46618c292a7f1918e351 . "./code.cg2js/cg-string-dialog.fasl")
(#xdbd7278e34d37f4ae735c7e6f462b1e8 . "./code.cg2js/cg-list-view.fasl")
(#xfc9e5e05e62495ca1f5452db6dd786e4 . "./code.cg2js/cg-intersect.fasl")
(#xa82a0d2346b70b675b362ec5f3bec6b8 . "./code.cg2js/cg-text-widget.fasl")
(#x9e51262edd402e118c51fe6d053a7d4f . "./code.cg2js/cg-directory-dialog.fasl")
(#xd1fc49592d90145726506b6451e4b007 . "./code.cg2js/cg-file-dialog.fasl")
(#x433f42e8991e01308e8341010d0ca750 . "./code.cg2js/cg-dde-utils.fasl")
(#x80007f7987b1bdc86eab62680d65af40 . "./code.cg2js/cg-nodes.fasl")
(#x4b5e07022979e8c4b9d0ec162effdf16 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xa2bda2cf9b9c25d979ad804fcaa74915 . "./code.cg2js/cg-up-down-control.fasl")
(#x351147cc7b0cde02580d79562360176b . "./code.cg2js/cg-get-position.fasl")
(#xc326053c9d7cf61a4ff4a5d18c19169c . "./code.cg2js/cg-toggling-widget.fasl")
(#x0f71760fafedc5ef69986a313ece890a . "./code.cg2js/cg-acache.fasl")
(#x5eb5c00de0a5f36e52239ecf37a50c59 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#xfea0666689fcbed5b9f4a079b607a39a . "./code.cg2js/cg-base.fasl")
(#x4cd73049659b21c753cc38b0cca310ff . "./code.cg2js/cg-drag-and-drop.fasl")
(#xeda8c25740b289026bcb49f2f301329b . "./code.cg2js/cg-find-string-dialog.fasl")
(#x0eae450450cb21e6aede3ab80e17fe08 . "./code.cg2js/cg-property.fasl")
(#x12562029404bc8c620afcfcc11422d8a . "./code.cg2js/cg-editable-text.fasl")
(#x7560eed1bc56dcd0a070c9e2b1bf4224 . "./code.cg2js/cg-lamp.fasl")
(#x86aa56320026481d94bc3906606fe1e2 . "./code.cg2js/cg-progress-indicator.fasl")
(#xd08aefe966dd257657f58e94e8358e77 . "./code.cg2js/cg-chart-widget.fasl")
(#xefa9ec6a1104149e64bf75e1a4ddc925 . "./code.cg2js/cg-common-control.fasl")
(#x4bfc3f1ff6b0cbeb02020120e3deb6ab . "./code.cg2js/cg-font-dialog.fasl")
(#x55fb8199f79e206dd48f0edfe0853041 . "./code.cg2js/cg-gesture-emulation.fasl")
(#xbdc34e1bc4c8c0c39b9f9c458015e505 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#xd85306a1c3931bc16ee11c540495cbe8 . "./code.cg2js/cg-os-window.fasl")
(#x8f82a757b560a15c53d5fac311ff808a . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x4285b7cada57cdcd4490453a2df12a6e . "./code.cg2js/cg-pixmap.fasl")
(#xeac1c120fbccff20ba332c0226ffd574 . "./code.cg2js/cg-chart-or-plot.fasl")
(#x81efd38ca5588ea885122c09f97d9931 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x944e66277a8d70ec9ccae8e999fe3cc2 . "./code.cg2js/cg-scaling-stream.fasl")
(#x4b647a54985674bb03d67db1d59ff0c4 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#xf4a541c7eed4b83b84dc4dd56d591e10 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#xe88de4e833faaad730fbfa7efee16562 . "./code.cg2js/cg-timer.fasl")
(#x7065d88b780807a2744b7ac61ac66d83 . "./code.cg2js/cg-scroll-bar.fasl")
(#xa3a6a94adf899372d1618b5d4d5d6ab2 . "./code.cg2js/cg-caret.fasl")
(#xe67599b82b9989a1482810c191dcaca7 . "./code.cg2js/cg-template-string.fasl")
(#x90e39428e3dee2a533bac9e38a4b7cc3 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x86deba9dd591ca34fb7ac9158702eb45 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x53cec2bfaa270dfd23737c469ea9020f . "./code.cg2js/cg-rich-edit.fasl")
(#x642d91ac09496117abf1cb5ba913de16 . "./code.cg2js/cg-combo-box.fasl")
(#x77cf4d5beb6995125f2bf57ba8b2816e . "./code.cg2js/cg-text-edit-pane.fasl")
(#x924260b310d13ca6749f7a51b4cb72ba . "./code.cg2js/cg-multi-picture-button.fasl")
(#x795008c66abe686b1e56649149061006 . "./code.cg2js/cg-menu-tooltip.fasl")
(#x81a9c2286be6d22a092c302e01725759 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x24f8e5e8a6b58b19c51243efe71a2480 . "./code.cg2js/cg-outline.fasl")
(#x56cf0187967c44baa24f8bebda61aa79 . "./code.cg2js/cg-class-support.fasl")
(#x7d55c6e30d46e6dc331fc16b316b8fee . "./code.cg2js/cg-header-control.fasl")
(#x3ae2bb93b0c0835ab69710c72e29a54a . "./code.cg2js/cg-lettered-menu.fasl")
(#x2ca8ab8b8e43874f0a488fd586171c92 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#xb50c76953405af4a268c024a18f25a1d . "./code.cg2js/cg-fill-texture.fasl")
(#x4c281c8974880a12342bae849c87f9de . "./code.cg2js/cg-lisp-widget.fasl")
(#xed512935ceb1464abc0455de33f778cf . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x4826e5130d2837cb876eb548551960bf . "./code.cg2js/cg-clipboard.fasl")
(#x4a1b44c67a64851417278bc201ce4cd4 . "./code.cg2js/cg-wrap-string.fasl")
(#x2d82f1df13e86fb8ba5f24d3ecaa0552 . "./code.cg2js/cg-os-widget.fasl")
(#xb8bbba7a8bae2cb410bcca343e91ecc8 . "./code.cg2js/cg-check-box.fasl")
(#x9d17f57ff4dc8ba639d7d0b1f137bca3 . "./code.cg2js/cg-choice-list.fasl")
(#x162bdd7b85ab41bed23fb7661fce54b7 . "./code.cg2js/cg-layout.fasl")
(#x58c333c18f3da646bd19ae7558e77422 . "./code.cg2js/cg-group-box.fasl")
(#x5d5b4c4ee46afcaa15379cf3b097ce7c . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x36253ceab337a1820c3a4148d030e654 . "./code.cg2js/cg-radio-button.fasl")
(#x325be534005c46c4f3fd141f655149ee . "./code.cg2js/cg-tab-control.fasl")
(#x1d4dbe97196d3229b5fd634253671c0f . "./code.cg2js/cg.fasl")
(#x6e9261413142c70f5939c5a5ff640ec7 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xccd4bcbe75d0400ae327299d09882920 . "./code.cg2js/cg-dropping-outline.fasl")
(#x1140c0d41295480867b9bed35132339e . "./code.cg2js/cg-get-pixmap.fasl")
(#xe3d17ca1a7bf1ae13d710aca3c5eb987 . "./code.cg2js/cg-grid-widget.fasl")
(#xac1bb4470d3ecfa9e628731cfc0bc47d . "./code.cg2js/cg-utility-dialog.fasl")
(#x385dd5d18f0a3dc591b0bb4bbb3dfb2c . "./code.cg2js/cg-static-text.fasl")
(#x8fef497195af72fb3c7d1db8cc0a6baf . "./code.cg2js/cg-class-slot-grid.fasl")
(#xf06a1d7cff4a100b81b341e02ad2a35d . "./code.cg2js/cg-object-editor-layout.fasl")
(#x61cb41df10e425298282baa4e39692b0 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#xf97dede3e2530c90f3b7f52c4b4ab577 . "./code.cg2js/cg-bitmap-pane.fasl")
(#x23af6e591c4d1e18f8b5014824569152 . "./code.cg2js/cg-plot-widget.fasl")
(#xadca56d013bab683e7fc69b1974659be . "./code.cg2js/cg-color-dialog.fasl")
(#x7da018d9ff981fc962d927fa2bd20a43 . "./code.cg2js/cg-icon-pixmap.fasl")
(#x2914d099b245e48b0451e5d526adbf0f . "./code.cg2js/cg-icon.fasl")
(#x4e1898a0e8b7bfa7ddc94dc4731eb569 . "./code.cg2js/cg-dev.fasl")
(#xe84ae87e7384375140d5645424da367e . "./code.cg2js/cg-choose-printer.fasl")
(#x33944f4b1bf7052fff59030dccbcac1b . "./code.cg2js/cg-intersect-posbox.fasl")
(#x26527f1a6b65d2d3b558d0e0ea23d2b8 . "./code.cg2js/cg-menu.fasl")
(#x8ade19e100bf25418837f0d44bd6e0a1 . "./code.cg2js/cg-yes-no-list.fasl")
(#xfdfa1cda4e6172115f780879c60960ea . "./code.cg2js/cg-palette.fasl")
(#xff0275a3e091d16571c82354e79726e8 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x140447a8929699c0dea838b632412879 . "./code.cg2js/cg-message-dialog.fasl")
(#x49cad226b562ad3a8c8eb47304edb6a9 . "./code.cg2js/cg-shortcut-menu.fasl")
(#x0dc2f80e40284fdbcd42d4d067a63c77 . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#xc72776d7b8e35534cef44dd8f0e85a28 . "./code.cg2js/cg-split-bar.fasl")
(#xc4bcb2585b5375a9e966cac6f80cd0d6 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x2108a692021999ad0895b66786ca2c08 . "./code.cg2js/cg-web-browser.fasl")
(#x8824c94a8b5839b177c7a3961085198b . "./code.cg2js/cg-project-window.fasl")
(#xdc3f882a8cd01b85a31f5fe1a96f3cf1 . "./code.cg2js/cg-picture-widget.fasl")
(#x357bb4c4dd31c79bc838d8d351b9f256 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x631bec3afe2ea6fec976db0a10ee9d39 . "./code.cg2js/cg-object-editor.fasl")
(#x825d738072a5c9bebf28ab7752b20b87 . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#xbf3c8adedd4c5bfc0016496e47450e5c . "./code.cg2js/cg-paren-matching.fasl")
(#x21074fdd61840497d2bb2269f4922d88 . "./code.cg2js/cg-sample-file-menu.fasl")
(#x401bbaffe1586bac9f7bce5beb284cfd . "./code.cg2js/cg-drawable.fasl")
(#xc8755086b861334cc0e8b936d842b1c8 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#xaf2c0119492ae4514c1a04665570ce7f . "./code.cg2js/cg-html-widget.fasl")
(#x89c84bcb57725c4af44a261f39fb4436 . "./code.cg2js/cg-curve.fasl")
(#x7657f1572ec5c68122c9fb35701849ca . "./code.cg2js/cg-yes-no-string.fasl")
(#xc19054e50c3223edf4882214f3dc839d . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x24b2e501c6d19b97c6f18fc5191c078b . "./code.cg2js/cg-html-dialog.fasl")
(#x8c18a0657ad757899fa72fb27b0b145e . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x453403563e3af1d7e1c0f610bb546c66 . "./code.cg2js/cg-tray.fasl")
(#xb58ccc9c5aa19118063899c76997c97f . "./code.idejs/ide-apropos.fasl")
(#x2e042837e4562f48cbfd734770ad66f8 . "./code.idejs/ide-window-list.fasl")
(#xb6928b273f0ac3076fca5ba2e299df4d . "./code.idejs/ide-class-browser.fasl")
(#x25ba34b3d33fe71df4097403cfe6c169 . "./code.idejs/ide-base.fasl")
(#x8189847da5880aa527861343101f6634 . "./code.idejs/ide-process-browser.fasl")
(#x81cc062377492b8147b394bbe892f366 . "./code.idejs/ide-acache.fasl")
(#x779df8d5a07db3b36e4411aa49a8c1b0 . "./code.idejs/ide-navigator.fasl")
(#xa955236fdabffb134f0158882f52de40 . "./code.idejs/ide-trace.fasl")
(#x9ebd08480464ad89e4c54660685627db . "./code.idejs/ide-project.fasl")
(#x56552f5a4d840a05344ef0e73ef33259 . "./code.idejs/ide-patch-dialog.fasl")
(#x7f110ea1568ea6e34b8c968be49ffc4a . "./code.idejs/ide-grapher.fasl")
(#xbe8599e0abe27f250b19ee467b188246 . "./code.idejs/ide-dev.fasl")
(#x393f623be87654bb09995116a66ce23b . "./code.idejs/ide-keyshort.fasl")
(#x59c45c166dc0a5fb1eaf2ff7edce57f3 . "./code.idejs/ide-editor.fasl")
(#x272d74133d6dcce5988dda773dbf1418 . "./code.idejs/ide-inspector.fasl")
(#x4afb342eee0b5ab81ffdd765a95175aa . "./code.idejs/ide.fasl")
(#xbb3be7548cae1b5444fe577bd49126ad . "./code.idejs/ide-clipboard.fasl")
(#xe438876a56be64c6582a45a45740674e . "./code.idejs/ide-find-definitions.fasl")
(#x07db2c50672c26b2e40d35e386ab88c8 . "./code.idejs/ide-form.fasl")
(#x0977c18b882627c63eac40092cec38d1 . "./code.idejs/ide-intro.fasl")
(#x2d32c8e7ebb687c69ff82dd5ba1d66c5 . "./code.idejs/ide-find-in-files.fasl")
(#x7ffafb5e7183700a2e17b701e4b09674 . "./code.idejs/ide-package-list.fasl")
(#x7a37bea088671554322a516bfc0a3bf2 . "./code.idejs/ide-class-grapher.fasl")
(#xb19a66ec109be5a69fb5293c577fc104 . "./cgjs.html")
(#x34779e5d4bbb71d45bf63f88fac54100 . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js/" "code.idejs/"))
    (format s "
(let ((path (make-pathname
	     :type \"fasl\"
	     :directory (pathname-directory
			 (merge-pathnames
			  ~s
			  (translate-logical-pathname \"sys:\"))))))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
